from typing import Callable
from sqlalchemy import create_engine, orm, MetaData
from sqlalchemy.orm import Session


class Database:
    """Database session manager"""

    def __init__(self, db_url: str, sa_future: bool = False, sa_echo: bool = False,
                 sa_session_autocommit=False, sa_session_autoflush=False) -> None:
        self._database_created = False
        self._engine = create_engine(
            db_url,
            pool_pre_ping=True,
            echo=sa_echo,
            future=sa_future)
        self._session_factory = orm.sessionmaker(
            autocommit=sa_session_autocommit,
            autoflush=sa_session_autoflush,
            bind=self._engine)

    def create_database(self, metadata: MetaData, start_mappers: Callable) -> None:
        if not self._database_created:
            metadata.create_all(self._engine, checkfirst=True)
            start_mappers()
            self._database_created = True

    def session(self) -> Session:
        return self._session_factory()



from sqlalchemy.orm import reconstructor
from propzen.common.domain.entity import Entity

@reconstructor
def sqlalchemy_entity_constructor(self):
    self.events = []

Entity.orm_constructor = sqlalchemy_entity_constructor



import time
from typing import Protocol, runtime_checkable, Type
from functools import wraps
from dataclasses import dataclass, field
from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class ExternalEvent:
    """Base class for external events."""

    timestamp: float = field(
        default_factory=time.time,
        init=False, compare=False,
    )

    @classmethod
    def routingkey(cls):
        return cls.__name__


@runtime_checkable
class IExternalPublisher(Protocol):
    """Interface for external publisher"""

    def publish(self, event: ExternalEvent):
        ...


class ExternalEventHandler:
    """Decorator helper to cache mapping of functions as external event handlers"""

    def __init__(self):
        self.mapping = {}

    def subscribe(self, eventtype: Type[ExternalEvent]):
        """Subscribes decorated function as handler for eventtype

        Parameters
        ----------
        eventtype: Type[ExternalEvent]
            ExternalEvent type to handle
        """

        def decorate(handler):
            self.mapping[eventtype] = handler

            @wraps(handler)
            def wrapper_func(*args, **kwargs):
                return handler(*args, **kwargs)
            return wrapper_func
        return decorate



from uuid import UUID
from dataclasses import dataclass
from propzen.common.service_layer.externalbus import ExternalEvent


@dataclass
class ProfilePictureUploaded(ExternalEvent):
    account_id: UUID
    filename: str


@dataclass
class PropertyPictureUploaded(ExternalEvent):
    account_id: UUID
    property_id: UUID
    filename: str


@dataclass
class AssetAttachmentUploaded(ExternalEvent):
    account_id: UUID
    asset_id: UUID
    file_id: UUID
    filename: str
    originalname: str


@dataclass
class AssetImageUploaded(ExternalEvent):
    account_id: UUID
    asset_id: UUID
    file_id: UUID
    filename: str
    originalname: str



class DomainBaseException(Exception):
    def __init__(self, service_code, type_code, exc_type='DomainException', message=None):
        self.service_code = service_code
        self.type_code = type_code
        self.exc_type = exc_type
        self.message = message
        super().__init__(self.message)

    @property
    def code(self):
        return f'{self.service_code}-{self.type_code}'


class DomainException(DomainBaseException):
    def __init__(self, service_code='0', type_code='0', exc_type='DomainException', message=None):
        super().__init__(service_code, type_code, exc_type, message)


class EntityNotFound(DomainBaseException):
    def __init__(self, service_code='0', type_code='1', exc_type='EntityNotFound', message=None):
        super().__init__(service_code, type_code, exc_type, message)



import time
from dataclasses import dataclass, field
from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class Event:
    timestamp: float = field(
        default_factory=time.time,
        init=False,
        compare=False,
    )



from contextlib import contextmanager
import pika


class Connection:
    """Connection Manager for Rabbit MQ using pika library."""

    exchange: str = 'propzen.direct'

    def __init__(self, bus_url: str) -> None:
        """
        Parameters
        ----------
        bus_url : str
            RabbitMQ URL parameters for pika.URLParameters
        """

        self.connection_params = pika.URLParameters(bus_url)
        self.connection = None
        self.channel = None

    @contextmanager
    def connect(self):
        """Contextually establishes a connection and opens a channel which disposes on exit

        Returns
        -------
        pika.channel.Channel
            yields a connected pika Channel
        """

        self.open_channel()
        yield self.channel
        self.close()

    def open_channel(self):
        """Establishes a connection and returns an open channel.

        Returns
        -------
        pika.channel.Channel
            a connected pika Channel
        """

        if self.connection is None or not self.connection.is_open:
            self.connection = pika.BlockingConnection(self.connection_params)
        self.channel = self.connection.channel()
        return self.channel

    def close(self):
        """Close and disposes current channel and connection."""

        if self.channel is not None:
            if self.channel.is_open:
                self.channel.close()
            self.channel = None

        if self.connection is not None:
            if self.connection.is_open:
                self.connection.close()
            self.connection = None



import re
from setuptools import setup
import setuptools

with open('README.md', mode='r', encoding='utf-8') as readmefile:
    long_description = readmefile.read()

with open("src/propzen/__init__.py", mode='r', encoding='utf-8') as init_file:
    version = re.search("__version__ = \"(.*?)\"", init_file.read()).group(1)


setup(
    name='propzen-common',
    version=version,
    author='Mikhail Christian Peralta',
    author_email='mikhail.peralta@gmail.com',
    maintainer='Mikhail Christian Peralta',
    maintainer_email='mikhail.peralta@gmail.com',
    description='Common package for PropZen',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://pypi.python.org/pypi/propzen-common',
    package_dir={'': 'src'},
    packages=setuptools.find_packages(where='src'),
    python_requires='>=3.9',
    install_requires=[
        'pydantic',
        'python-dotenv',
        'sqlalchemy',
        'dataclasses-json',
        'pika',
        'tenacity',
    ],
    zip_safe=True,
    license='MIT',
)



from uuid import UUID
from dataclasses import dataclass
from propzen.common.service_layer.externalbus import ExternalEvent


@dataclass
class AccountRegistered(ExternalEvent):
    fullname: str
    email: str
    token: str


@dataclass
class AccountConfirmed(ExternalEvent):
    account_id: UUID
    fullname: str
    email: str


@dataclass
class ResetPasswordRequested(ExternalEvent):
    fullname: str
    email: str
    token: str
